<?php
/*
  Plugin Name: Audio Comparison
  Plugin URI: https://audiocomparison.kaedinger.de/
  Description: Time synchronized A/B comparison of up to three audio files side by side. In style. For detailed documentation, please visit the plugin's <a href="https://audiocomparison.kaedinger.de/"> official page</a>.
  Version: 3.1
  Author: kaedinger
  Author URI: https://kaedinger.de
*/
defined( 'ABSPATH' ) || exit;
if (!function_exists('AC_ACL_write_log')) {
  function AC_ACL_write_log($comp, $log) {
    if (true === WP_DEBUG) {
      if (is_array($log) || is_object($log)) {
          $log = print_r($log, true);
      }
      error_log("$comp: $log");
    }
  }
}
require_once (dirname( __FILE__ ) . '/styling.php');
require_once (dirname( __FILE__ ) . '/promotion-checker.php');
if( ! class_exists( 'audioComparison' ) ) {
  class audioComparison {
    const SHORTCODE = 'audiocomparison';
    const MAIN_DIV = 'audio-comparison';
    const OPTIONS_KEY = 'audiocomparison_options';
    const SETTINGS_TITLE = 'Audio Comparison';
    const LITE = '';
    const STYLER = 'audioComparisonStyling';
    const PROMOTER = 'audioComparisonPromotionChecker';
    const MY_VERSION_NUMBER = '3.1';
    private $Styling;
    private $Promoter;
    private function write_log($log) { AC_ACL_write_log(self::SETTINGS_TITLE, $log); }
    public function __construct() {
      $styling_class = self::STYLER;
      $this->Styling = new $styling_class(self::MAIN_DIV);
      $promoter_class = self::PROMOTER;
      $this->Promoter = new $promoter_class();
      add_action( 'wp_enqueue_scripts', array($this, 'enqueue_scripts' ) );
      add_action( 'init', array($this, 'init' ) );
      add_action('admin_menu', array($this, 'create_menu' ) );
      add_action('admin_init', array($this, 'admin_init' ));
      $current_options = $this->get_options();
      if($current_options['apply_style_safely'] == "on") {
        add_action( 'wp_head', [ $this, 'insert_style_in_header' ] );
      }
    }
    public function insert_style_in_header() {
      $current_options = $this->get_options();
      if($current_options['apply_style'] == "on") {
        echo "<!-- " . self::SETTINGS_TITLE . " styling -->";
        $Theme = $current_options['style_theme'];
        $ColA = $current_options['style_color_a'];
        $ColB = $current_options['style_color_b'];
        $ColC = $current_options['style_color_c'];
        $Width = $current_options['style_width'];
        $Height = $current_options['style_height'];
        $Border = $current_options['style_border'];
        $Corner = $current_options['style_corner'];
        $FontSize = $current_options['style_font_size'];
        $LabelColor = $current_options['style_label_color'];
        $LabelWidth = $current_options['style_label_width'];
        echo "<style>" . $this->Styling->generate_css($Theme,$ColA,$ColB,$ColC,$Width,$Height,$Border,$Corner,$FontSize,$LabelColor,$LabelWidth) . "</style>";
      }
    }
    public function enqueue_scripts() {
      wp_enqueue_script('jquery');
      wp_enqueue_script('howler-js', plugins_url('lib/howler.core.min.js', __FILE__), self::MY_VERSION_NUMBER);
      wp_enqueue_script('audiocomparison_md5_js', plugins_url("/lib/jquery.md5.min.js", __FILE__), array('jquery'), self::MY_VERSION_NUMBER);
      wp_enqueue_script('audiocomparison_license_js', plugins_url("license.js", __FILE__), array('audiocomparison_md5_js'), self::MY_VERSION_NUMBER);
      wp_enqueue_script('audio-comparison', plugins_url('audio-comparison.js', __FILE__), array('audiocomparison_license_js','howler-js'), self::MY_VERSION_NUMBER);
    }
    public function init() {
      add_shortcode(self::SHORTCODE, array($this, 'shortcode' ) );
    }
    public function shortcode($attr = [], $content = null, $tag = '') {
      $current_options = $this->get_options();
      if($current_options['apply_style_safely'] != "on") {
        add_action( 'wp_head', [ $this, 'insert_style_in_header' ] );
      }
      $plugin_data = $this->get_plugin_data();
      $opts = $this->get_options();
      if ($opts['text_ready'] === "debug")
      {
        $buff = '<h3>Audio Comparison is in debug mode!</h3>';
        $buff .= '<h4>plugin_data</h4>';
        $buff .= '<p><pre>' . print_r($plugin_data, true) . '</pre></p>';
        $buff .= '<h4>opts</h4>';
        $buff .= '<p><pre>' . print_r($opts, true) . '</pre></p>';
      }
      $attr = array_change_key_case( (array) $attr, CASE_LOWER );
      if ($opts['text_ready'] === "debug")
      {
        $buff .= '<h4>lowercase attr</h4>';
        $buff .= '<p><pre>' . print_r($attr, true) . '</pre></p>';
      }
      $attr = shortcode_atts(
        array(
          'play_button' => $opts['play_button'],
          'text_button_play' => $opts['text_button_play'],
          'text_button_stop' => $opts['text_button_stop'],
          'text_a' => $opts['text_a'],
          'text_b' => $opts['text_b'],
          'text_c' => $opts['text_c'], 
          'text' => $opts['text'],
          'text_loading' => $opts['text_loading'],
          'text_ready' => $opts['text_ready'],
          'text_button_a' => $opts['text_button_a'],
          'text_button_b' => $opts['text_button_b'],
          'text_button_c' => $opts['text_button_c'], 
          'before' => '',
          'file_a' => '',
          'file_b' => '',
          'file_c' => '', 
          'play_default' => $opts['play_default'],
        ), $attr, $tag
      );
      if ($opts['text_ready'] === "debug")
      {
        $buff .= '<h4>shortcode_atts</h4>';
        $buff .= '<p><pre>' . print_r($attr, true) . '</pre></p>';
        return $buff;
      }
      $endl = "\n";
      $playbuttonbuff = '';
      $playbuttontextsbuff = '';
      $playButton = esc_html($attr['play_button']);
      if($playButton != "none")
      {                        
        $playDefault = esc_html($attr['play_default']);
        $playbuttontextsbuff .= ' data-play-default="' . $playDefault . '"';
        $textPlay = $attr['text_button_play'];
        $textStop = $attr['text_button_stop'];
        $playbuttontextsbuff .= ' data-button-play-text="' . $textPlay . '"';
        $playbuttontextsbuff .= ' data-button-stop-text="' . $textStop . '"';
        $playbuttonbuff .= '<button class="' . self::MAIN_DIV . '-play-stop">' . html_entity_decode($textPlay) . '</button>' . $endl;
      }                        
      $fileA = esc_html($attr['file_a']);
      $fileB = esc_html($attr['file_b']);
      $fileC = esc_html($attr['file_c']);   
      $buff = '';
      $buff .= "\n<!-- {$plugin_data['name']} | {$plugin_data['url']} -->\n";
      $buff .= '<div class="' . self::MAIN_DIV . '"';
      $text = esc_html($attr['text']);
      if($text != "none")
      {
        $textA = $attr['text_a'];
        if(!empty($textA)) $buff .= ' data-playing-a-text="' . $textA . '"';
        if(!empty($fileB)) {
          $textB = $attr['text_b'];
          if(!empty($textB)) $buff .= ' data-playing-b-text="' . $textB . '"';
        }
        if(!empty($fileC)) {
          $textC = $attr['text_c'];
          if(!empty($textC)) $buff .= ' data-playing-c-text="' . $textC . '"';
        }
        $textLoading = $attr['text_loading'];
        if(!empty($textLoading)) $buff .= ' data-buffering-text="' . $textLoading . '"';
        $textReady = $attr['text_ready'];
        if(!empty($textReady)) $buff .= ' data-buffered-text="' . $textReady . '"';
      }
      $buff .= $playbuttontextsbuff . ">\n";
      $buff .= $attr['before'];
      $label = '<span class="' . self::MAIN_DIV . '-label"></span>' . $endl;
      switch($text)
      {
        case "before";
        $buff .= $label;
        case "none";
        $label = '';
      }
      if(!empty($fileA)) $buff .= '<a class="' . self::MAIN_DIV . '-file-a" href="' . $fileA . '"></a>' . $endl;
      if(!empty($fileB)) $buff .= '<a class="' . self::MAIN_DIV . '-file-b" href="' . $fileB . '"></a>' . $endl;
      if(!empty($fileC)) $buff .= '<a class="' . self::MAIN_DIV . '-file-c" href="' . $fileC . '"></a>' . $endl;
      if($playButton == "left")
      {
        $buff .= $playbuttonbuff;
        $playbuttonbuff = '';
      }
      $buttonA = $attr['text_button_a'];
      $buff .= '<button class="' . self::MAIN_DIV . '-play-a">' . html_entity_decode($buttonA) . '</button>' . $endl;
      if(!empty($fileB)) {
        $buttonB = $attr['text_button_b'];
        $buff .= '<button class="' . self::MAIN_DIV . '-play-b">' . html_entity_decode($buttonB) . '</button>' . $endl;
      }
      if(!empty($fileC)) {
        $buttonC = $attr['text_button_c'];
        $buff .= '<button class="' . self::MAIN_DIV . '-play-c">' . html_entity_decode($buttonC) . '</button>' . $endl;
      }
      $buff .= $playbuttonbuff;
      $buff .= $label;
      $buff .= "</div>";
      $buff .= "<!-- ^^^ {$plugin_data['name']} | {$plugin_data['url']} -->\n";
      return $buff;
    }
    public function get_plugin_data() {
      $default_headers = array(
          'Name' => 'Plugin Name',
          'PluginURI' => 'Plugin URI',
      );
      $plugin_data = get_file_data(__FILE__, $default_headers, 'plugin');
      $url = $plugin_data['PluginURI'];
      $name = $plugin_data['Name'];
      $data['name'] = $name;
      $data['url'] = $url;
      return $data;
    }
    public function get_defaults() {
      return array(
        'text' => 'before',            
        'play_button' => 'left',       
        'text_a' => 'Now playing A',
        'text_b' => 'Now playing B',
        'text_c' => 'Now playing C',
        'text_loading' => 'Buffering audio...',
        'text_ready' => 'Ready!',
        'text_button_play' => 'Play',
        'text_button_stop' => 'Stop',
        'text_button_a' => 'Version A',
        'text_button_b' => 'Version B',
        'text_button_c' => 'Version C',
        'play_default' => 'A',         
        'style_theme' => 'ac', 
        'style_color_a' => '#f0a07c',
        'style_color_b' => '#4a274f',
        'style_color_c' => '#ffffff',
        'style_width' => '140',
        'style_height' => '10',
        'style_border' => '2',
        'style_corner' => '0',
        'style_font_size' => '13',
        'style_label_color' => '#000000',
        'style_label_width' => '200',
        'apply_style' => '',
        'apply_style_safely' => '',
        'tryout_backgroundcolor' => '#f0f0f1',
        'tryout_3rdbutton' => ''
      );
    }
    public function get_options() {
      $defaults = $this->get_defaults();
      $current_options = get_option( self::OPTIONS_KEY, $defaults);
      if ($current_options['text_ready'] === "reset") {
        $current_options = $defaults;
      } else {
        $current_options = array_merge($defaults, $current_options);
      }
      return $current_options;
    }
    public function set_options($opts) {
      foreach ($opts as $key => $value) {
          $value = wp_kses($value, array());
          $value = trim($value);
          $opts[$key] = $value;
      }
      update_option( self::OPTIONS_KEY , $opts);
      return $opts;
    }
    public function create_menu() {
      add_options_page(self::SETTINGS_TITLE, self::SETTINGS_TITLE, 'manage_options', __FILE__, array($this, 'settings_page' ) );
      add_filter('plugin_action_links', array($this, 'add_plugin_settings_link' ), 10, 2);
      add_filter( 'plugin_row_meta', [ $this, 'add_plugin_row_meta_link' ], 10, 2 );
    }
    public function add_plugin_row_meta_link($links, $file) {
      if ($file == plugin_basename(__FILE__)) {
        $doc_link = '<a href="https://audiocomparison.kaedinger.de/docs" target="_blank" rel="noopener noreferrer">Documentation</a>';
        $link = admin_url('options-general.php?page=' . plugin_basename(__FILE__));
        $pro_link = "<a href=\"{$link}\" 
        style=\"background-color: #f0a07c; color: #4a274f; font-weight: 700\" 
        onmouseover=\"this.style.color='#f0a07c'; this.style.backgroundColor='#4a274f';\" 
        onmouseout=\"this.style.color='#4a274f'; this.style.backgroundColor='#f0a07c';\" 
        >&nbsp;Activate your Pro license!&nbsp;</a>";
        $doc_link .= "<span style=\"visibility: hidden;\" id=\"audiocomparison_activate_pro\"> | {$pro_link}</span>";
        $links[] = $doc_link;
      }
      return $links;
    }
    public function add_plugin_settings_link($links, $file) {
      if ($file == plugin_basename(__FILE__)) {
        $link = admin_url('options-general.php?page=' . plugin_basename(__FILE__));
        $dashboard_link = "<a href=\"{$link}\">Settings</a>";
        array_unshift($links, $dashboard_link);
      }
      return $links;
    }
    public function load_settings_assets() {
      wp_enqueue_script('jquery');
      $themes = $this->Styling->get_theme_templates();
      $localized = [
        'MAIN_DIV' => self::MAIN_DIV,
        'themes' => $themes,
      ];
      wp_enqueue_script('audiocomparison_styler_js', plugins_url("styling.js", __FILE__), array('jquery'), self::MY_VERSION_NUMBER);
      wp_localize_script('audiocomparison_styler_js', 'AUDIOCOMPARISON', $localized);
    }
    public function settings_textinput($for, $disabled = false) { ?>
      <input maxlength="60" id="<?=$for?>" name="<?=$for?>" autocomplete="off" value="<?php echo esc_attr($this->get_options()[$for]); ?>"<?php echo $disabled?" disabled":"" ?> />
    <?php }
    public function settings_page() {
      $this->load_settings_assets();
      $saved = 0;
      $current_options = $this->get_options();
      $themes = $this->Styling->get_theme_templates();
      if (!empty($_POST)) 
      {
        $current_options_keys = array_keys($current_options);
        $_REQ = stripslashes_deep($_REQUEST);
        if ($current_options['text_ready'] === "debug") {
          print_r($_REQ);
        }
        foreach ($_REQ as $key => $value) {
          if (in_array($key, $current_options_keys)) {
            $value = esc_attr(wp_kses_post($value));
            $value = trim($value);
            $current_options[$key] = $value;
          }
        }
        $current_options['apply_style'] = array_key_exists('apply_style', $_REQ) ? "on" : "";
        $current_options['apply_style_safely'] = array_key_exists('apply_style_safely', $_REQ) ? "on" : "";
        $current_options = $this->set_options($current_options);
        $saved = 1;
      }
    ?>
    <div class="wrap <?php echo self::SHORTCODE; ?>_container">
      <style>
      .<?php echo self::SHORTCODE; ?>_settings th {
        text-align: right;
      }
      .<?php echo self::SHORTCODE; ?>_settings select {
        width: 28ch;
      }
      .<?php echo self::SHORTCODE; ?>_settings input[type=submit], input[type=color], input[type=range] {
        width: 20ch;
      }
      .<?php echo self::SHORTCODE; ?>_settings img {
        width: 200px;
      }
      .AC_ROW { display: flex; }
      .AC_ROW a { text-decoration: none; }
      .AC_COL50 { flex: 50%; }
      @media screen and (orientation: portrait) { .AC_COL { flex: 100%; } }
      </style>
      <?php if (!empty($saved)) : ?>
        <h2></h2>
        <div class="updated">
          <p>Settings were saved.
          </p>
        </div>
      <?php endif; ?>
      <div class="AC_ROW">
        <div style="flex: 0 0 110px;">
          <img src="<?php echo plugins_url('/logo.png', __FILE__); ?>" style="width: 100px">
        </div>  
        <div style="flex: 1;">
          <p><span style="font-size: x-large"><?php echo self::SETTINGS_TITLE; ?></span> <span style="font-size: small"> Version 3.1</span></p>
          <p>
            Time synchronized A/B comparison <?php echo self::LITE; ?>of up to three audio files side by side. In style.<br>
            Developed by <a href="https://studio.kaedinger.de">studio kaedinger</a>
            &bull; <a href="https://audiocomparison.kaedinger.de/docs/">Documentation</a>
            &bull; <a href="mailto:support@kaedinger.de?subject=SUPPORT%20-%20Audio%20Comparison%20on%20<?php echo $_SERVER['SERVER_NAME'] ?>">Contact Support</a>
          </p>  
        </div>  
        <div style="flex: none; text-align: right;">
          <?php
            $mylinks = $this->Promoter->get_links();
            if(count($mylinks) > 0) {
              echo implode("<br/>", $mylinks);
            }
          ?>
        </div>  
      </div>  
      <form method="post">
        <div class="AC_ROW">
          <div class="AC_COL50">
            <h2>Settings</h2>
            <table class="<?php echo self::SHORTCODE; ?>_settings">
              <tr>
                <th scope="row"><label for="text">Output:</label></th>
                <td><select id="text" name="text">
                  <option value="before"<?php echo wp_kses_post( $current_options['text'] == "before" ? " selected" : "" ); ?>>Before the buttons</option>
                  <option value="after"<?php echo wp_kses_post( $current_options['text'] == "after" ? " selected" : "" ); ?>>After the buttons</option>
                  <option value="none"<?php echo wp_kses_post( $current_options['text'] == "none" ? " selected" : "" ); ?>>No text output</option>
                </select></td>
              </tr>
              <tr>
                <th scope="row"><label for="text_a">Output for A<sup>1</sup>:</label></th>
                <td><?php $this->settings_textinput('text_a'); ?> e.g. 'Now playing A'</td>
              </tr>
              <tr>
                <th scope="row"><label for="text_b">Output for B<sup>1</sup>:</label></th>
                <td><?php $this->settings_textinput('text_b'); ?> e.g. 'Now playing B'</td>
              </tr>
              <tr>
                <th scope="row"><label for="text_c">Output for C<sup>1</sup>:</label></th>
                <td><?php $this->settings_textinput('text_c'); ?> e.g. 'Now playing C'</td>
              </tr>
              <tr>
                <th scope="row"><label for="text_loading">Output during file load:</label></th>
                <td><?php $this->settings_textinput('text_loading'); ?> e.g. 'Buffering audio...'</td>
              </tr>
              <tr>
                <th scope="row"><label for="text_ready">Output after file load:</label></th>
                <td><?php $this->settings_textinput('text_ready'); ?> e.g. 'Ready!'</td>
              </tr>
              <tr>
                <th scope="row"><label for="play_button">Play/Stop button:</label></th>
                <td><select id="play_button" name="play_button">
                      <option value="left"<?php echo wp_kses_post( $current_options['play_button'] == "left" ? " selected" : "" ); ?>>Left of the A/B/C buttons</option>
                      <option value="right"<?php echo wp_kses_post( $current_options['play_button'] == "right" ? " selected" : "" ); ?>>Right of the A/B/C buttons</option>
                      <option value="none"<?php echo wp_kses_post( $current_options['play_button'] == "none" ? " selected" : "" ); ?>>No play button</option>
                </select></td>
              </tr>
              <tr>
                <th scope="row"><label for="play_default">Play default:</label></th>
                <td><select id="play_default" name="play_default">
                      <option value="A"<?php echo wp_kses_post( $current_options['play_default'] == "A" ? " selected" : "" ); ?>>Play A on default</option>
                      <option value="B"<?php echo wp_kses_post( $current_options['play_default'] == "B" ? " selected" : "" ); ?>>Play B on default</option>
                      <option value="C"<?php echo wp_kses_post( $current_options['play_default'] == "C" ? " selected" : "" ); ?>>Play C on default</option>
                    </select>
                </td>
              </tr>
              <tr>
                <th scope="row"><label for="text_button_play">Label on Play/Stop button for Play:</label></th>
                <td><?php $this->settings_textinput('text_button_play'); ?> e.g. 'Play'</td>
              </tr>
              <tr>
                <th scope="row"><label for="text_button_stop">Label on Play/Stop button for Stop:</label></th>
                <td><?php $this->settings_textinput('text_button_stop'); ?> e.g. 'Stop'</td>
              </tr>
              <tr>
                <th scope="row"><label for="text_button_a">Label on A button:</label></th>
                <td><?php $this->settings_textinput('text_button_a'); ?> e.g. 'Version A'</td>
              </tr>
              <tr>
                <th scope="row"><label for="text_button_b">Label on B button:</label></th>
                <td><?php $this->settings_textinput('text_button_b'); ?> e.g. 'Version B'</td>
              </tr>
              <tr>
                <th scope="row"><label for="text_button_c">Label on C button:</label></th>
                <td>
                  <?php $this->settings_textinput('text_button_c'); ?> e.g. 'Version C'
                </td>
              </tr>
              <tr>
                <td>
                  <br><input type="submit" name="save_settings" value="Save Changes" class="button-primary" />
                </td>
              </tr>
            </table>
          </div>
          <div class="AC_COL50">
            <style id="ACSTYLE"></style>
            <h2>Styling</h2>
            <table class="<?php echo self::SHORTCODE; ?>_settings">
              <tr>
                <th scope="row"><label for="style_theme">Theme:</label></th>
                <td colspan="4">
                  <select id="style_theme" name="style_theme">
                    <?php
                    foreach (['ac','sk','ss','ep'] as $t) {
                      $d = $themes[$t . '_defaults'];
                      $n = $d['Name'];
                      $p = $d['Pro'];
                      echo "<option value=\"$t\" " . ( $current_options['style_theme'] == $t ? " selected" : "" ) . ">$n</option>";
                    }
                    ?>
                  </select>
                  <button type="button" id="reset_defaults" class="button-primary">Reset to defaults</button>
                  <span id="clicktoreset" style="visibility: hidden;">&nbsp;Kept your changes. Click to restore the theme defaults.</span>
                </td>
              </tr>
              <tr>
                <th scope="row"><label for="style_color_a">Color A:</label></th>
                <td><input type="color" id="style_color_a" name="style_color_a" value="<?php echo esc_attr($current_options['style_color_a']); ?>"></td>
                <th scope="row"><label for="style_font_size">Font size:</label></th>
                <td><input type="range" id="style_font_size" name="style_font_size" min="10" max="40" value="<?php echo esc_attr($current_options['style_font_size']); ?>"></td>
              </tr>
              <tr>
                <th scope="row"><label for="style_color_b">Color B:</label></th>
                <td><input type="color" id="style_color_b" name="style_color_b" value="<?php echo esc_attr($current_options['style_color_b']); ?>"></td>
                <th scope="row"><label for="style_label_color">Label color:</label></th>
                <td><input type="color" id="style_label_color" name="style_label_color" value="<?php echo esc_attr($current_options['style_label_color']); ?>"></td>
              </tr>
              <tr>
                <th scope="row"><label for="style_color_c">Color C:</label></th>
                <td><input type="color" id="style_color_c" name="style_color_c" value="<?php echo esc_attr($current_options['style_color_c']); ?>"></td>
                <th scope="row"><label for="style_label_width">Label width:</label></th>
                <td><input type="range" id="style_label_width" name="style_label_width" min="30" max="300" value="<?php echo esc_attr($current_options['style_label_width']); ?>"></td>
              </tr>
              <tr>
                <th scope="row"><label for="style_width">Width:</label></th>
                <td><input type="range" id="style_width" name="style_width" min="50" max="250" value="<?php echo esc_attr($current_options['style_width']); ?>"></td>
              </tr>
              <tr>
                <th scope="row"><label for="style_height">Height:</label></th>
                <td><input type="range" id="style_height" name="style_height" min="5" max="100" value="<?php echo esc_attr($current_options['style_height']); ?>"></td>
              </tr>
              <tr>
                <th scope="row"><label for="style_border">Border:</label></th>
                <td><input type="range" id="style_border" name="style_border" min="0" max="20" value="<?php echo esc_attr($current_options['style_border']); ?>"></td>
              </tr>
              <tr>
                <th scope="row"><label for="style_corner">Corner:</label></th>
                <td><input type="range" id="style_corner" name="style_corner" min="0" max="50" value="<?php echo esc_attr($current_options['style_corner']); ?>"></td>
              </tr>
              <tr>
                <td colspan="2">
                  <input type="checkbox" id="apply_style" name="apply_style"<?php echo $current_options['apply_style'] == "on" ? " checked" : ""; ?>><label for="apply_style" style="font-weight: bold"> Apply style on whole site automatically</label><br>
                  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<input type="checkbox" id="apply_style_safely" name="apply_style_safely"<?php echo $current_options['apply_style_safely'] == "on" ? " checked" : ""; ?>><label for="apply_style_safely" style="font-weight: bold"> Use safe method</label>
                </td>
                <td colspan="2">
                  <button type="button" id="copy_to_clipboard" class="<?php echo self::SHORTCODE; ?>_settings button-primary">Copy style to clipboard</button>
                </td>
              </tr>
            </table>
            <p class="<?php echo self::SHORTCODE; ?>_settings">
              <input type="submit" name="save_settings" value="Save Changes" class="button-primary" />
            </p>
            <div class="tryoutarea <?php echo self::SHORTCODE; ?>_settings">
              <p>
                <div class="<?php echo self::MAIN_DIV; ?>">
                  <span class="<?php echo self::MAIN_DIV; ?>-label <?php echo self::MAIN_DIV; ?>-label--before <?php echo self::MAIN_DIV; ?>-label-ready">Ready!</span>
                  <button class="<?php echo self::MAIN_DIV; ?>-play-stop <?php echo self::MAIN_DIV; ?>-play-stop--left" type="button">Play/Stop</button>
                  <button class="<?php echo self::MAIN_DIV; ?>-play-a" type="button">Version A</button>
                  <button class="<?php echo self::MAIN_DIV; ?>-play-b" type="button">Version B</button>
                  <button class="<?php echo self::MAIN_DIV; ?>-play-c" type="button">Version C</button>  
                  <button class="<?php echo self::MAIN_DIV; ?>-play-stop <?php echo self::MAIN_DIV; ?>-play-stop--right" type="button">Play/Stop</button>
                  <span class="<?php echo self::MAIN_DIV; ?>-label <?php echo self::MAIN_DIV; ?>-label--after <?php echo self::MAIN_DIV; ?>-label-ready">Ready!</span>
                </div>
              <p>
                <div class="<?php echo self::MAIN_DIV; ?>">
                  <span class="<?php echo self::MAIN_DIV; ?>-label <?php echo self::MAIN_DIV; ?>-label--before <?php echo self::MAIN_DIV; ?>-output-buffering">Buffering...</span>
                  <button class="<?php echo self::MAIN_DIV; ?>-play-stop <?php echo self::MAIN_DIV; ?>-play-stop--left <?php echo self::MAIN_DIV; ?>-button-buffering" disabled="disabled">Play/Stop</button>
                  <button class="<?php echo self::MAIN_DIV; ?>-play-a <?php echo self::MAIN_DIV; ?>-button-buffering" disabled="disabled">Version A</button>
                  <button class="<?php echo self::MAIN_DIV; ?>-play-b <?php echo self::MAIN_DIV; ?>-button-buffering" disabled="disabled">Version B</button>
                  <button class="<?php echo self::MAIN_DIV; ?>-play-c <?php echo self::MAIN_DIV; ?>-button-buffering" disabled="disabled">Version C</button>  
                  <button class="<?php echo self::MAIN_DIV; ?>-play-stop <?php echo self::MAIN_DIV; ?>-play-stop--right <?php echo self::MAIN_DIV; ?>-button-buffering" disabled="disabled">Play/Stop</button>
                  <span class="<?php echo self::MAIN_DIV; ?>-label <?php echo self::MAIN_DIV; ?>-label--after <?php echo self::MAIN_DIV; ?>-output-buffering">Buffering...</span>
                </div>
              <p>
                <div class="<?php echo self::MAIN_DIV; ?>">
                  <span class="<?php echo self::MAIN_DIV; ?>-label <?php echo self::MAIN_DIV; ?>-label--before <?php echo self::MAIN_DIV; ?>-output-playing-a">Playing A</span>
                  <button class="<?php echo self::MAIN_DIV; ?>-play-stop <?php echo self::MAIN_DIV; ?>-play-stop--left <?php echo self::MAIN_DIV; ?>-playing" type="button">Play/Stop</button>
                  <button class="<?php echo self::MAIN_DIV; ?>-play-a <?php echo self::MAIN_DIV; ?>-button-playing" disabled="disabled">Version A</button>
                  <button class="<?php echo self::MAIN_DIV; ?>-play-b" type="button">Version B</button>
                  <button class="<?php echo self::MAIN_DIV; ?>-play-c" type="button">Version C</button>  
                  <button class="<?php echo self::MAIN_DIV; ?>-play-stop <?php echo self::MAIN_DIV; ?>-play-stop--right <?php echo self::MAIN_DIV; ?>-playing" type="button">Play/Stop</button>
                  <span class="<?php echo self::MAIN_DIV; ?>-label <?php echo self::MAIN_DIV; ?>-label--after <?php echo self::MAIN_DIV; ?>-output-playing-a">Playing A</span>
                </div>
              <p>
                <div class="<?php echo self::MAIN_DIV; ?>">
                  <span class="<?php echo self::MAIN_DIV; ?>-label <?php echo self::MAIN_DIV; ?>-label--before <?php echo self::MAIN_DIV; ?>-output-playing-b">Playing B</span>
                  <button class="<?php echo self::MAIN_DIV; ?>-play-stop <?php echo self::MAIN_DIV; ?>-play-stop--left <?php echo self::MAIN_DIV; ?>-playing" type="button">Play/Stop</button>
                  <button class="<?php echo self::MAIN_DIV; ?>-play-a" type="button">Version A</button>
                  <button class="<?php echo self::MAIN_DIV; ?>-play-b <?php echo self::MAIN_DIV; ?>-button-playing" disabled="disabled">Version B</button>
                  <button class="<?php echo self::MAIN_DIV; ?>-play-c">Version C</button>  
                  <button class="<?php echo self::MAIN_DIV; ?>-play-stop <?php echo self::MAIN_DIV; ?>-play-stop--right <?php echo self::MAIN_DIV; ?>-playing" type="button">Play/Stop</button>
                  <span class="<?php echo self::MAIN_DIV; ?>-label <?php echo self::MAIN_DIV; ?>-label--after <?php echo self::MAIN_DIV; ?>-output-playing-b">Playing B</span>
                </div>
                <p>
            </div>
            <label for="tryout_backgroundcolor">Tryout background color:&nbsp;</label><input type="color" id="tryout_backgroundcolor" name="tryout_backgroundcolor" style="margin-top: 5px;" value="<?php echo esc_attr($current_options['tryout_backgroundcolor']); ?>">
            <input type="checkbox" id="tryout_3rdbutton" name="tryout_3rdbutton" style="margin-left: 20px;" <?php echo $current_options['tryout_3rdbutton'] == "on" ? " checked" : ""; ?>><label for="tryout_3rdbutton"> Show button C</label>  
          </div>
        </div>
      </form>
      <h2>License</h2>
      <p>License server says: <strong><span id="audiocomparison_license">(still asking...)</span></strong></p>
      <p>License keys can be activated on one server at a time. You can activate and deactivate this server from here.</p>
        <p>Enter your license key: <input maxlength="80" id="audiocomparison_licensekey" name="licensekey" autocomplete="off" value="" style="width: 40ch" /> 
        <br><br><input type="submit" id="audiocomparison_enterlicense" name="enterlicense" value="Activate" class="button-primary" onclick="AC_LicenseEntered()" style="width: 20ch" />
        &nbsp;&nbsp;&nbsp;&nbsp;<span id="audiocomparison_response"></span></p>
<style>
  .doc_hide { display: none; }
  #doc_show:checked ~ label + .doc_hide { display: block; }
  #doc_show:checked ~ label > a { display: none; }
</style>
<input id="doc_show" type="checkbox" style="display: none">
<label for="doc_show"><a>Click to show documentation</a></label>
<div class="doc_hide">
      <h2>Shortcode</h2>
      <p>The base version of <?php echo self::SETTINGS_TITLE; ?>'s shortcode is <code>[<?php echo self::SHORTCODE; ?> file_a="PATH/TO/AUDIO.A" file_b="PATH/TO/AUDIO.B"]</code><br>
      This way, all settings from above are used. It's a great way to keep a consistent look throughout your website for all instances of <?php echo self::SETTINGS_TITLE; ?>.<br>
      (Since version 3 you can add <code>file_c="PATH/TO/AUDIO.C"</code> to compare three files 
      and you can only use <code>file_a</code> if you want the simplest WordPress Audio Player ever!)</p>
      <p>However, you can give any instance an individual UI. Any option from above is also available as a shortcode attribute. 
        Any attribute not specified in the shortcode will use the site-wide setting from above.</p>
      <table>
      <tr>
          <td><code>text</code></td>
          <td><strong>Output</strong>. Sets the position of the output label. Options: <code>before</code> (before the buttons), <code>after</code> (after the buttons),
          <code>none</code> (no output label).
          Default: <code>before</code></td>
        </tr>
        <tr>
          <td><code>text_a</code></td>
          <td><strong>Output for A</strong><sup>1</sup>. 
          Contents of output label while playing audio A. Default: <code>Now playing A</code></td>
        </tr>
        <tr>
          <td><code>text_b</code></td>
          <td><strong>Output for B</strong><sup>1</sup>. 
          Contents of output label while playing audio B. Default: <code>Now playing B</code></td>
        </tr>
        <tr>
          <td><code>text_c</code></td>
          <td><strong>Output for C</strong><sup>1</sup>. 
          Contents of output label while playing audio C. Default: <code>Now playing C</code></td>
        </tr>
        <tr>
          <td><code>text_loading</code></td>
          <td><strong>Output during file load</strong>. 
          Contents of output label while buffering audio. Default: <code>Buffering audio...</code></td>
        </tr>
        <tr>
          <td><code>text_ready</code></td>
          <td><strong>Output after file load</strong>. 
          Contents of output label after audio has been loaded. Default: <code>Ready!</code></td>
        </tr>
        <tr>
          <td><code>play_button</code></td>
          <td><strong>Play/Stop button</strong>. Sets the position of the Play/Stop button. Options: <code>left</code> (left of the sound buttons), <code>right</code> (right of the sound buttons), 
          <code>none</code> (no Play/Stop button). 
          Default: <code>left</code></td>
        </tr>
        <tr>
          <td><code>play_default</code></td>
          <td><strong>Play on default</strong>. What sound to play after pressing the Play button. Options: <code>A</code> (sound A), 
          <code>B</code> (sound B), <code>C</code> (sound C).
          Default: <code>A</code></td>
        </tr>
        <tr>
          <td><code>text_button_play</code></td>
          <td><strong>Label on Play/Stop button for Play</strong>. Play/Stop button label while not playing audio. Default: <code>Play</code></td>
        </tr>
        <tr>
          <td><code>text_button_stop</code></td>
          <td><strong>Label on Play/Stop button for Stop</strong>. Play/Stop button label while playing audio. Default: <code>Stop</code></td>
        </tr>
        <tr>
          <td><code>text_button_a</code></td>
          <td><strong>Label on A button</strong>. Text on button A. Default: <code>Version A</code></td>
        </tr>
        <tr>
          <td><code>text_button_b</code></td>
          <td><strong>Label on B button</strong>. Text on button B. Default: <code>Version B</code></td>
        </tr>
        <tr>
          <td><code>text_button_c</code></td>
          <td><strong>Label on C button</strong>. Text on button C. Default: <code>Version C</code></td>
        </tr>
      </table>
      <h2>Notes</h2>
      <p>You can use HTML in the label and button texts, like '&lt;i>Play&lt;/i>'.</p>
      <sup>1</sup> You can use the placeholder <code>%T</code> for putting the current position (format m:ss) in the text.<br>
      <h2>Design</h2>
      <p>You can use the following CSS classes to fine tune the visual style of the buttons and the label.</p>
      <h4>Static</h4>
      <table>
        <tr>
          <td><code>.<?php echo self::MAIN_DIV; ?></code></td>
          <td>The whole instance <code>div</code>.</td>
        </tr>
        <tr>
          <td><code>.<?php echo self::MAIN_DIV; ?>-play-stop</code></td>
          <td>Play/Stop button.</td>
        </tr>
        <tr>
          <td><code>.<?php echo self::MAIN_DIV; ?>-play-a</code></td>
          <td>Button A.</td>
        </tr>
        <tr>
          <td><code>.<?php echo self::MAIN_DIV; ?>-play-b</code></td>
          <td>Button B.</td>
        </tr>
        <tr>
          <td><code>.<?php echo self::MAIN_DIV; ?>-play-c</code></td>
          <td>Button C.</td>
        </tr>
        <tr>
          <td><code>.<?php echo self::MAIN_DIV; ?>-label</code></td>
          <td>Output label.</td>
        </tr>
      </table>
      <h4>Dynamic</h4>
      <table>
        <tr>
          <td><code>.<?php echo self::MAIN_DIV; ?>-output-buffering</code></td>
          <td>Assigned to the output label while audio is still loading.</td>
        </tr>
        <tr>
          <td><code>.<?php echo self::MAIN_DIV; ?>-output-playing-a</code></td>
          <td>Assigned to the output label while audio A is playing.</td>
        </tr>
        <tr>
          <td><code>.<?php echo self::MAIN_DIV; ?>-output-playing-b</code></td>
          <td>Assigned to the output label while audio B is playing.</td>
        </tr>
        <tr>
          <td><code>.<?php echo self::MAIN_DIV; ?>-output-playing-c</code></td>
          <td>Assigned to the output label while audio C is playing.</td>
        </tr>
        <tr>
          <td><code>.<?php echo self::MAIN_DIV; ?>-button-buffering</code></td>
          <td>Assigned to all buttons during audio loading.</td>
        </tr>
        <tr>
          <td><code>.<?php echo self::MAIN_DIV; ?>-button-playing</code></td>
          <td>Assigned to the A, B, or C button while the respective audio is playing.</td> 
        </tr>
        <tr>
          <td><code>.<?php echo self::MAIN_DIV; ?>-playing</code></td>
          <td>Assigned to the Play/Stop button while audio is playing.</td>
        </tr>
        <tr>
          <td><code>disabled="disabled"</code></td>
          <td >Buttons that cannot be clicked<br>(e.g. Play/Stop 
            button during buffering, button A while playing A, etc.)<br>
            have the <code>disabled</code> attribute set.</td>
        </tr>
      </table>
      <a href="https://audiocomparison.kaedinger.de/docs/" target="ACDOCS">Full documentation</a>
</div>
    </div>
      <?php
    }
    public function admin_init() {
      $this->load_assets();
    }
    public function load_assets() {
      wp_enqueue_script('jquery');
      wp_enqueue_script('audiocomparison_md5_js', plugins_url("/lib/jquery.md5.min.js", __FILE__), array('jquery',), self::MY_VERSION_NUMBER);
      wp_enqueue_script('audiocomparison_license_js', plugins_url("license.js", __FILE__), array('jquery','audiocomparison_md5_js'), self::MY_VERSION_NUMBER);
      wp_enqueue_script('audiocomparison_admin_js', plugins_url("admin.js", __FILE__), array('jquery','audiocomparison_license_js'), self::MY_VERSION_NUMBER);
    }
  }
  new audioComparison();
}
// auto update
if( ! class_exists( 'audioComparisonUpdateChecker' ) ) {
	class audioComparisonUpdateChecker{
		public $plugin_slug;
		public $version;
		public $cache_key;
		public $cache_allowed;
    public function write_log($log) { AC_ACL_write_log('audioComparisonUpdateChecker', $log); }
		public function __construct() {
			$this->plugin_slug = plugin_basename( __DIR__ );
			$this->version = '3.1';
			$this->cache_key = 'audio_comparison_upd';
      $this->cache_key_failed = 'audio_comparison_upd_failed';
			$this->cache_allowed = true;
			add_filter( 'plugins_api', array( $this, 'info' ), 20, 3 );
			add_filter( 'site_transient_update_plugins', array( $this, 'update' ) );
			add_action( 'upgrader_process_complete', array( $this, 'purge' ), 10, 2 );
		}
		public function request(){
      $failed_before = get_transient( $this->cache_key_failed );
      if( ! ( false === $failed_before ) ) {
        return false;
      }
			$remote = get_transient( $this->cache_key );
			if( false === $remote || ! $this->cache_allowed ) {
				$remote = wp_remote_get(
					'https://assets.kaedinger.de/static/AC/infoAC.json',
					array(
						'timeout' => 5,
						'headers' => array(
							'Accept' => 'application/json'
						)
					)
				);
				if(
					is_wp_error( $remote )
					|| 200 !== wp_remote_retrieve_response_code( $remote )
					|| empty( wp_remote_retrieve_body( $remote ) )
				) {
          set_transient( $this->cache_key_failed, "failed", DAY_IN_SECONDS / 2 );
					return false;
				}
				set_transient( $this->cache_key, $remote, DAY_IN_SECONDS );
			}
			$remote = json_decode( wp_remote_retrieve_body( $remote ) );
			return $remote;
		}
		function info( $res, $action, $args ) {
			if( 'plugin_information' !== $action ) {
				return $res;
			}
			if( $this->plugin_slug !== $args->slug ) {
				return $res;
			}
			$remote = $this->request();
			if( ! $remote ) {
				return $res;
			}
			$res = new stdClass();
			$res->name = $remote->name;
			$res->slug = $remote->slug;
			$res->version = $remote->version;
			$res->tested = $remote->tested;
			$res->requires = $remote->requires;
			$res->author = $remote->author;
			$res->author_profile = $remote->author_profile;
			$res->download_link = $remote->download_url;
			$res->trunk = $remote->download_url;
			$res->requires_php = $remote->requires_php;
			$res->last_updated = $remote->last_updated;
			$res->sections = array(
				'description' => $remote->sections->description,
				'installation' => $remote->sections->installation,
				'changelog' => $remote->sections->changelog
			);
			if( ! empty( $remote->banners ) ) {
				$res->banners = array(
					'low' => $remote->banners->low,
					'high' => $remote->banners->high
				);
			}
			return $res;
		}
		public function update( $transient ) {
			if ( empty($transient->checked ) ) {
				return $transient;
			}
			$remote = $this->request();
      if( $remote) {
        $ver = version_compare( $this->version, $remote->version, '<' );
        $blogver = get_bloginfo( 'version' );
        $blogv = version_compare( $remote->requires, $blogver, '<=' );
        $phpv = version_compare( $remote->requires_php, PHP_VERSION, '<=' );
        $this->write_log("Version $this->version < $remote->version: " . ($ver ? "yes" : "no") . ", WP $remote->requires <= $blogver: " . ($blogv ? "yes" : "no") . ", PHP $remote->requires_php <= " . PHP_VERSION . ": " . ($phpv ? "yes" : "no"));
			  if($ver && $blogv && $phpv) {
          $res = new stdClass();
          $res->slug = $this->plugin_slug;
          $res->plugin = plugin_basename( __FILE__ );
          $res->new_version = $remote->version;
          $res->tested = $remote->tested;
          $res->package = $remote->download_url;
          $transient->response[ $res->plugin ] = $res;
        }
	    }
			return $transient;
		}
		public function purge($upgrader, $options){
			if (
				$this->cache_allowed
				&& 'update' === $options['action']
				&& 'plugin' === $options[ 'type' ]
			) {
				delete_transient( $this->cache_key );
        delete_transient( $this->cache_key_failed );
			}
		}
	}
	new audioComparisonUpdateChecker();
}
